/******************************************************************************
*  Filename:       serialIO.c
*  Revised:        $ $
*  Revision:       $ $
*
*  Description:    Help functions for handling queues
*
*  Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
*
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************/

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

/* XDCtools Header files */
#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/System.h>

#include "smartrf_settings/smartrf_settings.h"
#include "serialIO.h"
#include "processCommand.h"

extern rfc_CMD_FS_t RF_cmdFs;
extern rfc_CMD_PROP_RADIO_DIV_SETUP_t RF_cmdPropRadioDivSetup;
extern rfc_CMD_TX_TEST_t RF_cmdTxTest;

extern RF_Handle rfHandle;
extern RF_CmdHandle cw_cmd;
extern RF_CmdHandle rx_cmd;
extern UART_Handle uart;

extern volatile uint16_t tx_numPackets;
extern volatile uint32_t pktIntervalTime;

extern uint16_t defaultFrequency;
extern uint16_t defaultFractFreq;

/* Char buffer to store absolute frequency */
char absFreqStr[10];

volatile enum rf_state rfState;

volatile int txFreqHoppingEnabled = 0;
volatile int rxFreqHoppingEnabled = 0;

int8_t command_process(){
    /* Update with new parameters*/
	if(uart_s.commandType == COMMAND_SET) {
    	return (command_set());
    }

	/* Read and generate response string */
	if(uart_s.commandType == COMMAND_GET) {
        command_get();
	}

	return 0;
}

//*****************************************************************************
//
//! Processes a query and prepares the response
//!
//
//*****************************************************************************
void command_get() {

	uart_s.responseMsgLen = 0;

    switch (uart_s.command) {
        case ABSOLUTE_FREQUENCY:
        	freqToString();
        	break;

        case RADIO_STATE:
        	break;

        case TX_POWER:
        	getTxPower();
        	break;

        case TX_DATARATE:
        	getDataRate();
			break;

        case RX_DATARATE:
        	break;

        case TX_BIT_SHAPING:
        	getBitShaping();
        	break;

        case FREQ_DEV:
        	getFreqDeviation();
        	break;

        case TX_PACKET:
        	getTxPacket();
        	break;

        case RX_FILTER_BW:
        	getRxFilterBW();
        	break;

        case CW:
        	break;

        case GET_RADIO_CONFIG:
        	break;

        case GET_RADIO_REGISTER:
        	break;

        case WRITE_REGISTER_FUNCTION:
        	break;


        default:
    	    strcpy(uart_s.response, "invalid command");
    	    uart_s.responseMsgLen = 15;
        	break;

	}

}

//*****************************************************************************
//
//! Processes a command and updates the smart RF settings
//!
//
//*****************************************************************************

int8_t command_set() {

	int8_t actionCommand = 0;

	uart_s.responseMsgLen = 0;

    switch (uart_s.command) {

        case USAGE:
            uart_usage();
    	    break;

        case ABSOLUTE_FREQUENCY:
        	set_absoluteFrequency();
        	break;

        case TX_REPEAT:
        	tx_numPackets = getNumPacket();
        	pktIntervalTime = getPacketInterval();
        	rfState = TX_TURN_ON;
        	actionCommand = 1;
        	break;

        case TX_POWER:
        	setTxPower();
        	break;

        case START_TX:
        	strcpy(uart_s.response, "  start tx received");
        	uart_s.responseMsgLen = 19;
        	rfState = TX_TURN_ON;
        	actionCommand = 1;
        	break;

        case START_TX_FH:
        	strcpy(uart_s.response, "  start tx freq hop received");
        	uart_s.responseMsgLen = 28;
        	rfState = TX_FH_ON;
        	txFreqHoppingEnabled = 1;
        	rxFreqHoppingEnabled = 0;
        	actionCommand = 1;
        	break;


        case STOP_TX:
        	strcpy(uart_s.response, "  stop tx received");
        	uart_s.responseMsgLen = 18;
        	rfState = TX_TURN_OFF;
        	actionCommand = 0;
        	txFreqHoppingEnabled = 0;
        	break;

        case START_RX:
        	strcpy(uart_s.response, "  start rx received");
        	uart_s.responseMsgLen = 19;
        	rfState = RX_TURN_ON;
        	actionCommand = 1;
        	break;

        case START_RX_FH:
        	strcpy(uart_s.response, "  start rx freq hop received");
        	uart_s.responseMsgLen = 28;
        	rfState = RX_FH_ON;
        	rxFreqHoppingEnabled = 1;
        	txFreqHoppingEnabled = 0;
        	actionCommand = 1;
        	break;

        case STOP_RX:
        	strcpy(uart_s.response, "  stop rx received");
        	uart_s.responseMsgLen = 18;
        	rfState = RX_TURN_OFF;
        	actionCommand = 0;
        	rxFreqHoppingEnabled = 0;
        	//RF_abortCmd(rfHandle, rx_cmd);
        	break;


        case CW:
        	if (strcmp(uart_s.arg[0], "on") == 0) {
        	    strcpy(uart_s.response, "  cw on received");
        	    uart_s.responseMsgLen = 16;
        	    RF_cmdTxTest.config.bUseCw = 1;
            	rfState = TX_CW_ON;
            	actionCommand = 1;

        	}

        	if (strcmp(uart_s.arg[0], "off") == 0) {
        	    strcpy(uart_s.response, "  cw off received");
        	    uart_s.responseMsgLen = 17;
        	    RF_cmdTxTest.config.bUseCw = 0;
            	actionCommand = 0;
        	    //RF_abortCmd(rfHandle, cw_cmd);
        	    rfState = TX_CW_OFF;
            	actionCommand = 1;
        	}

        	break;

        case GET_RADIO_CONFIG:
        	getRadioConfig();
        	break;

        case WRITE_REGISTER_FUNCTION:
        	break;

        case RESET:
            RF_cmdFs.frequency = defaultFrequency;
            RF_cmdPropRadioDivSetup.centerFreq = defaultFrequency;
            RF_cmdFs.fractFreq = defaultFractFreq;
            RF_close(rfHandle);
            rfHandle = NULL;
        	rfState = RX_OFF;
        	break;

        default:
    	    strcpy(uart_s.response, "  invalid command");
    	    uart_s.responseMsgLen = 17;
        	break;

	}

    return actionCommand;

}

//*****************************************************************************
//
//! Converts frequency into a string
//!
//
//*****************************************************************************

void freqToString(){
    int8_t i;
    uint16_t tmpFreq;
    uint32_t fractFreqHz;

    tmpFreq = RF_cmdFs.frequency;

    /* Grab RF_cmdFs.frequency */
    for (i = 3; i >= 0; i--) {
        absFreqStr[i] = (tmpFreq % 10) + CHAR_ZERO;
        tmpFreq = tmpFreq / 10;
    }

    /* Grab RF_cmdFs.fractFreq */
    fractFreqHz = (15625 * RF_cmdFs.fractFreq ) >> 10;
    for (i = 9; i >= 4; i--) {
        absFreqStr[i] = (fractFreqHz % 10) + CHAR_ZERO;
        fractFreqHz = fractFreqHz / 10;
    }

    strcpy(uart_s.response, "  absolute_frequency = ");
    strcat(uart_s.response, absFreqStr);
    uart_s.responseMsgLen = 33;

}

//*****************************************************************************
//
//! Updates the RF frequency setting as per user input
//!
//
//*****************************************************************************

void set_absoluteFrequency(){
    int8_t i;
    uint16_t frequency;
    uint64_t fractFreqHz;

    frequency = 0;
    for(i = 0; i< 4; i++) {
    	frequency = frequency * 10 + (uart_s.arg[0][i] - (char)CHAR_ZERO);
    }

    RF_cmdFs.frequency = frequency;
    RF_cmdPropRadioDivSetup.centerFreq = frequency;

    fractFreqHz = 0;
    for(i = 4; i< 10; i++) {
    	fractFreqHz = fractFreqHz * 10 + (uart_s.arg[0][i] - (char)CHAR_ZERO);
    }

    /* Convert frequency in Hz to format that the radio understands while retaining precision */
    RF_cmdFs.fractFreq = (fractFreqHz * 4295) >> 16;  // (65536/1M) <<  16 = 4294.96

    /* Close previous connection to radio */
    RF_close(rfHandle);
    rfHandle = NULL;

}

//*****************************************************************************
//
//! Sets the number of packets to transmit
//!
//
//*****************************************************************************
uint16_t getNumPacket() {
   uint16_t numPackets = 0;
   uint8_t i;

   for(i = 0; i< 4; i++) {
	   if ((uart_s.arg[0][i] >= (char)CHAR_ZERO) & (uart_s.arg[0][i] < (char)(CHAR_ZERO + 10))) {
	       numPackets = numPackets * 10 + (uart_s.arg[0][i] - (char)CHAR_ZERO);
	   }
   }

   return numPackets;

}


//*****************************************************************************
//
//! Sets the interval between transmit packets
//!
//
//*****************************************************************************
uint32_t getPacketInterval() {
    uint16_t intervalTime;
    uint8_t i;

    intervalTime = 0;

    for(i = 0; i< 3; i++) {
        if ((uart_s.arg[1][i] >= (char)CHAR_ZERO) & (uart_s.arg[1][i] < (char)(CHAR_ZERO + 10))) {
        	intervalTime = intervalTime * 10 + (uart_s.arg[1][i] - (char)CHAR_ZERO);
        }
    }

    if (intervalTime == 0)
    {
    	intervalTime = 500;  /* set to 500 ms */
    }

    return (intervalTime * 4000);

}


//*****************************************************************************
//
//! Reads the data rate into the uart response
//!
//
//*****************************************************************************
void getDataRate() {
    uint32_t rateword;
    uint16_t intRateword;
    uint16_t fractionalRateword;
    uint8_t i, value;


    rateword = RF_cmdPropRadioDivSetup.symbolRate.rateWord * 50;
    intRateword = rateword >>  15;
    fractionalRateword = rateword - (intRateword << 15);

    /* Convert fraction to two digit decimal value */
    fractionalRateword  = (fractionalRateword * 100 ) >> 15;

    /* Create response string */
    strcpy(uart_s.response, "  tx datarate =   0.00 kbps");

    /* Fill value before decimal point */
    i = 18;
    while(intRateword > 0) {
    	value = intRateword % 10;
    	uart_s.response[i] = value + (char)CHAR_ZERO;
    	intRateword = intRateword / 10;
    	i--;
    }

    /* Fill value after decimal point */
    i = 21;
    while(fractionalRateword > 0) {
    	value = fractionalRateword % 10;
    	uart_s.response[i] = value + (char)CHAR_ZERO;
    	fractionalRateword = fractionalRateword / 10;
    	i--;
    }

    uart_s.responseMsgLen = 27;

}

//*****************************************************************************
//
//! Reads the TX power in dB into the uart response
//!
//
//*****************************************************************************
#define TX_POWER_NUM_ENTRIES       15

const int8_t   txPowerTabledB[TX_POWER_NUM_ENTRIES] = {14, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, -10};
const uint16_t txPowerTable[TX_POWER_NUM_ENTRIES]   = {0xa73f, 0xb818, 0x50da, 0x38d3, 0x2ccd, 0x24cb, 0x20c9, 0x1cc7, 0x18c6, 0x18c5, 0x14c4, 0x1042, 0x10c3, 0x0041, 0x08c0};

void getTxPower() {
	uint8_t i;
	uint8_t units;

	for (i = 0; i< TX_POWER_NUM_ENTRIES; i++) {
		if (RF_cmdPropRadioDivSetup.txPower == txPowerTable[i])
			break;
	}

    /* Create response string */
    strcpy(uart_s.response, "  tx power =  0.00 dBm");

    if (abs(txPowerTabledB[i]) > 9)
        uart_s.response[13] = '1';

    if (txPowerTabledB[i] < 0)
    	uart_s.response[12] = '-';

    units = txPowerTabledB[i] % 10;
    uart_s.response[14] = units + (char)CHAR_ZERO;

    uart_s.responseMsgLen = 22;

}



//*****************************************************************************
//
//! Sets the TX power
//!
//
//*****************************************************************************
void setTxPower() {
	int i;
	int txPwrdB;

	txPwrdB = 0;


	if (uart_s.arg[0][0] == CHAR_MINUS) {
		/*Handle -ve numbers */
        for(i = 1; i< 3; i++) {
            if ((uart_s.arg[0][i] >= (char)CHAR_ZERO) & (uart_s.arg[0][i] < (char)(CHAR_ZERO + 10))) {
        	    txPwrdB = txPwrdB * 10 + (uart_s.arg[0][i] - (char)CHAR_ZERO);
            }
        }
        txPwrdB = -txPwrdB;
	}
	else {
        for(i = 0; i< 2; i++) {
            if ((uart_s.arg[0][i] >= (char)CHAR_ZERO) & (uart_s.arg[0][i] < (char)(CHAR_ZERO + 10))) {
        	    txPwrdB = txPwrdB * 10 + (uart_s.arg[0][i] - (char)CHAR_ZERO);
            }
        }
	}

	for (i = 0; i< TX_POWER_NUM_ENTRIES; i++) {
		if (txPwrdB == txPowerTabledB[i])
			break;
	}

	RF_cmdPropRadioDivSetup.txPower = txPowerTable[i];

	/* Get and print the value of the updated TX power */
	getTxPower();
}
//*****************************************************************************
//
//! Reads the if bit shaping is ON or OFF
//! Also reads the bit shaping setting 0, 0.5 or 1
//
//*****************************************************************************

void getBitShaping() {

	if (RF_cmdPropRadioDivSetup.modulation.modType == 0x01) {
	    strcpy(uart_s.response, "  tx_bit_shaping = ON 0.5");
	    uart_s.responseMsgLen = 25;

	}

	if (RF_cmdPropRadioDivSetup.modulation.modType == 0x00) {
	    strcpy(uart_s.response, "  tx_bit_shaping = OFF 0");
	    uart_s.responseMsgLen = 24;

	}


}


//*****************************************************************************
//
//! Reads the frequency deviation
//!
//
//*****************************************************************************
void getFreqDeviation() {
	uint8_t i;
	uint8_t freq_dev_Khz, value;

    /* Create response string */
    strcpy(uart_s.response, "  freq_dev =   0 KHz");
    freq_dev_Khz = RF_cmdPropRadioDivSetup.modulation.deviation >> 2;

    /* Fill value of frequency deviation in the response string */
    i = 15;
    while(freq_dev_Khz > 0) {
    	value = freq_dev_Khz % 10;
    	uart_s.response[i] = value + (char)CHAR_ZERO;
    	freq_dev_Khz = freq_dev_Khz / 10;
    	i--;
    }

    uart_s.responseMsgLen = 20;

}


//*****************************************************************************
//
//! Rreads and Prints the TX packet to UART
//!
//
//*****************************************************************************
void getTxPacket() {

	uart_writePayLoad(RF_cmdPropTx.pPkt, RF_cmdPropTx.pktLen);

}


//*****************************************************************************
//
//! Rreads and Prints the TX packet to UART
//!
//
//*****************************************************************************
#define RXBW_TABLESIZE    21
const uint16_t rxBWTable[RXBW_TABLESIZE] = { 45,  55,  66,  88,  110,  130,  180,  220,  260,  350,
                                      430, 530, 700, 870, 1060, 1410, 1740, 2120, 2800, 3470, 4240 };

void getRxFilterBW() {
    uint8_t i;
    uint16_t rxBW, value;

    rxBW = rxBWTable[RF_cmdPropRadioDivSetup.rxBw - 32];

    /* Create response string */
    strcpy(uart_s.response, "  rx_filter_bw =    0 KHz");


    /* Fill value of frequency deviation in the response string */
    i = 20;
    while(rxBW > 0) {
    	value = rxBW % 10;
    	uart_s.response[i] = value + (char)CHAR_ZERO;
    	rxBW = rxBW / 10;
    	i--;
    }

    uart_s.responseMsgLen = 25;

}

void getRadioConfig() {

	freqToString();
	printResponse();
	getDataRate();
	printResponse();
	getBitShaping();
	printResponse();
	getTxPower();
	printResponse();
	getFreqDeviation();
	printResponse();
	getRxFilterBW();


}

void printResponse() {
	char input;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    input = (char)CHAR_LINEFEED;
    UART_write(uart, &input, 1);
}


