/*
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/***** Includes *****/
#include <stdlib.h>
#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>

/* Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>

/* Board Header files */
#include "Board.h"

#define ENABLE_RX

#ifdef  ENABLE_RX
#include "RFQueue.h"
#endif

#include "smartrf_settings/smartrf_settings.h"

#include "serialIO.h"

#include "processCommand.h"

void createAndSendPacket();
void processRFState();
void runTxFreqHopping();
void runRxFreqHopping();

/* Pin driver handle */
static PIN_Handle ledPinHandle;
static PIN_State ledPinState;

/*
 * Application LED pin configuration table:
 *   - All LEDs board LEDs are off.
 */
PIN_Config pinTable[] =
{
    Board_LED3 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
    Board_LED4 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
	Board_KEY_RIGHT  | PIN_INPUT_EN | PIN_PULLUP | PIN_IRQ_NEGEDGE,
	Board_KEY_LEFT   | PIN_INPUT_EN | PIN_PULLUP | PIN_IRQ_NEGEDGE,
    PIN_TERMINATE
};


/***** Defines *****/
#define TX_TASK_STACK_SIZE 1024
#define TX_TASK_PRIORITY   3


/* TX Configuration */
#define NUMBER_OF_PACKETS       10
#define PAYLOAD_LENGTH          27
#define PACKET_INTERVAL 	4000000*0.5f

/* RX Configuration */
#define DATA_ENTRY_HEADER_SIZE 8  /* Constant header size of a Generic Data Entry */
#define MAX_LENGTH             30 /* Max length byte the radio will accept */
#define NUM_DATA_ENTRIES       2  /* NOTE: Only two data entries supported at the moment */
#define NUM_APPENDED_BYTES     3  /* The Data Entries data field will contain:*/



/***** Prototypes *****/
static void txTaskFunction(UArg arg0, UArg arg1);



/***** Variable declarations *****/
static Task_Params txTaskParams;
static Task_Struct txTask;
static uint8_t txTaskStack[TX_TASK_STACK_SIZE];

static RF_Object rfObject;
RF_Handle rfHandle;
RF_CmdHandle cw_cmd;
RF_CmdHandle rx_cmd;

UART_Handle uart;
UART_Params uartParams;

uint32_t time;
static uint8_t packet[PAYLOAD_LENGTH];
static uint16_t seqNumber;
static uint32_t packetCounter;
static PIN_Handle pinHandle;
extern volatile enum rf_state rfState;
extern volatile int txFreqHoppingEnabled;
extern volatile int rxFreqHoppingEnabled;


volatile uint16_t tx_numPackets = NUMBER_OF_PACKETS;
volatile uint32_t pktIntervalTime = PACKET_INTERVAL;


#if defined(__TI_COMPILER_VERSION__)
#pragma DATA_ALIGN (rxDataEntryBuffer, 4);
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma data_alignment = 4
#endif
static uint8_t rxDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                                 MAX_LENGTH,
                                                                 NUM_APPENDED_BYTES)];

/* Receive dataQueue for RF Core to fill in data */
static dataQueue_t dataQueue;
static rfc_dataEntryGeneral_t* currentDataEntry;
static uint8_t packetLength;
static uint8_t* packetDataPointer;

static uint8_t* rssiDataPointer;
int8_t rssi;
uint8_t error;
uint8_t prevRcvOK;
uint8_t rcvSeqNumber = 0;
uint8_t prevRcvSeqNumber = 0;

/* The RX Output struct contains statistics about the RX operation of the radio. */
static rfc_propRxOutput_t rxStatistics;
static PIN_Handle pinHandle;

uint16_t defaultFrequency;
uint16_t defaultFractFreq;

#define NUM_CHANNELS  46
typedef struct
{
  uint16_t frequency;
  uint16_t fractFreq;
} channel_Freq_T;

const channel_Freq_T fh_868_channels[NUM_CHANNELS] =
{
    {863,  9696}, {863, 19524}, {863, 29352}, {863, 39179}, {863, 49007}, {863, 58835}, {864,  3128}, {864, 12956},
    {864, 22784}, {864, 32611}, {864, 42439}, {864, 52267}, {864, 62095}, {865,  6388}, {865, 16216}, {865, 26044},
    {865, 35871}, {865, 45699}, {865, 55527}, {865, 65355}, {866,  9648}, {866, 19476}, {866, 29304}, {866, 39131},
    {866, 48959}, {866, 58787}, {867,  3080}, {867, 12908}, {867, 22736}, {867, 32563}, {867, 42391}, {867, 52219},
    {867, 62047}, {868,  6340}, {868, 16168}, {868, 25996}, {868, 35823}, {868, 45651}, {868, 55479}, {868, 65307},
    {869,  9600}, {869, 19428}, {869, 29256}, {869, 39083}, {869, 48911}, {869, 58739}

};

/***** Function definitions *****/
void TxTask_init(PIN_Handle inPinHandle)
{
    pinHandle = inPinHandle;

    Task_Params_init(&txTaskParams);
    txTaskParams.stackSize = TX_TASK_STACK_SIZE;
    txTaskParams.priority = TX_TASK_PRIORITY;
    txTaskParams.stack = &txTaskStack;
    txTaskParams.arg0 = (UInt)1000000;

    Task_construct(&txTask, txTaskFunction, &txTaskParams, NULL);
}

RF_Params rfParams;
RF_Event result;

static void txTaskFunction(UArg arg0, UArg arg1)
{

    RF_Params_init(&rfParams);

    RF_cmdPropTx.pktLen = PAYLOAD_LENGTH;
    RF_cmdPropTx.pPkt = packet;
    RF_cmdPropTx.startTrigger.triggerType = TRIG_ABSTIME;
    RF_cmdPropTx.startTrigger.pastTrig = 1;
    RF_cmdPropTx.startTime = 0;

    if( RFQueue_defineQueue(&dataQueue,
                            rxDataEntryBuffer,
                            sizeof(rxDataEntryBuffer),
                            NUM_DATA_ENTRIES,
                            MAX_LENGTH + NUM_APPENDED_BYTES))
    {
        /* Failed to allocate space for all data entries */
        while(true);
    }

    /* 4.1. Modify CMD_PROP_RX command for application needs */
    RF_cmdPropRx.pQueue = &dataQueue;               /* Set the Data Entity queue for received data */
    RF_cmdPropRx.pOutput = (uint8_t*)&rxStatistics;

#if 0
    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);

    /* Set the frequency */
    RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL);

#endif

    rfState = TX_TURN_OFF;

    defaultFrequency = RF_cmdFs.frequency;
    defaultFractFreq = RF_cmdFs.fractFreq;

    while(1)
    {
    	Semaphore_pend(semaphore_txTask, BIOS_WAIT_FOREVER);

    	/* Process the RF state just once */
    	if (!txFreqHoppingEnabled & !rxFreqHoppingEnabled) {
    	    processRFState();
    	}

    	/* Process the RF TX state by updating the frequency */
    	if (txFreqHoppingEnabled) {
        	runTxFreqHopping();
        }

    	/* Process the RF RX state by updating the frequency */
        if (rxFreqHoppingEnabled) {
        	runRxFreqHopping();
        }

    }
}

/*
 *  ======== main ========
 */
int main(void)
{
    /* Call board init functions. */
    Board_initGeneral();
    Board_initUART();


    /* Open LED pins */
    ledPinHandle = PIN_open(&ledPinState, pinTable);
    if(!ledPinHandle)
    {
        System_abort("Error initializing board LED pins\n");
    }

    /* Clear LED pins */
    PIN_setOutputValue(ledPinHandle, Board_LED3, 0);
    PIN_setOutputValue(ledPinHandle, Board_LED4, 0);

    /* Initialize task */
    TxTask_init(ledPinHandle);

    /* Start BIOS */
    BIOS_start();

    return (0);
}



/*
 *  ======== echoFxn ========
 *  Task for this function is created statically. See the project's .cfg file.
 */


Void echoFxn(UArg arg0, UArg arg1)
{
    char input;
    uint8_t charIndex;

    /* Create a UART with data processing off. */
    UART_Params_init(&uartParams);
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.readDataMode = UART_DATA_BINARY;
    uartParams.readReturnMode = UART_RETURN_FULL;
    uartParams.readEcho = UART_ECHO_OFF;
    uartParams.baudRate = 115200;
    uart = UART_open(Board_UART0, &uartParams);

    if (uart == NULL) {
        System_abort("Error opening the UART");
    }


    /* Loop forever echoing */
    charIndex = 0;

    uart_usage();

    while (1) {

        UART_read(uart, &input, 1);

    	/* Skip the line feed that appears after an enter as the first character of new line */
        if (input == (char)CHAR_LINEFEED)
        	charIndex--;


        if ((input == (char)CHAR_LINE_END_1) | (input == (char)CHAR_LINE_END_2)) // (charIndex < UART_SERIAL_LINE_SIZE))
        {
            uart_s.lineSize = charIndex;

            /* Parse the input into command and arguments */
            uart_get_command_and_arguments();

            /* Process the commands recived */
            /* If command_process returns 1, then process txTaskFunction */
            if(command_process())
                Semaphore_post(semaphore_txTask);

            /* Print the response of the command processing */
            UART_write(uart, uart_s.response, uart_s.responseMsgLen);

            input = (char)CHAR_LINEFEED;
            UART_write(uart, &input, 1);

            /* reset index to zero to point to begining of the line */
            charIndex = 0;
        }
        else
        {
        	/* Store the input character */
            uart_s.inputLine[charIndex++] = input;
        }

    }
}


void createAndSendPacket() {
	uint32_t time;

    /****** STEP 4 - TX ********************************************************
    * 1. Create packet (with increasing sequence number and random content)
    * 2. Set absolute TX time to utilize automatic power management
    * 3. Send CMD_PROP_TX command with blocking RF driver call
    * 4. (Go to step 1. until NUMBER_OF_PACKETS packets sent)
    ****************************************************************************/
    packetCounter = 0;
    time = RF_getCurrentTime();
    seqNumber = 0;

    while (packetCounter++ < tx_numPackets)
    //while (1)
    {
        /* 1. Create packet with incrementing sequence number and random payload */
        packet[0] = (uint8_t)(seqNumber >> 8);
        packet[1] = (uint8_t)(seqNumber++);
        uint8_t i;

        for (i = 2; i < PAYLOAD_LENGTH; i++)
        {
            packet[i] = i;
        }

        /* 2. Set absolute TX time to utilize automatic power management */
        time += pktIntervalTime;
        RF_cmdPropTx.startTime = time;

        /* 3. Send packet */
        if (rfState == TX_ON) {
            RF_Event result = RF_runCmd(rfHandle, (RF_Op*)&RF_cmdPropTx, RF_PriorityNormal);
            if (result != RF_EventCmdDone)
            {
                /* Error */
                while(1);
            }
            else
            {
        	    /* Done with TX; Print the payload */
        	    uart_writePayLoad(RF_cmdPropTx.pPkt, RF_cmdPropTx.pktLen);
            }
        }
        else {
        	break;
        }

        PIN_setOutputValue(pinHandle, Board_LED3,!PIN_getOutputValue(Board_LED3));
    }

    /* Close connection to radio */
    RF_close(rfHandle);
    rfHandle = NULL;

    return;
}


void processRFState() {


	if ((rfState == TX_TURN_ON) || (rfState == RX_TURN_ON))
	{

		if (rfHandle == NULL) {
		    /* Request access to the radio */
		    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);

		    /* Set the frequency */
		    RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL);
		}

		if (rfState == TX_TURN_ON)
            rfState = TX_ON;
		if (rfState == RX_TURN_ON)
			rfState = RX_ON;

	}

	if (rfState == TX_ON)
	{
        /****** STEP 4 - TX ********************************************************/
		createAndSendPacket();
	}

	if (rfState == TX_TURN_OFF)
	{
        /* Switch to off state */
        rfState = TX_OFF;
        seqNumber = 0;
        UART_writeCancel(uart);
	}

	if (rfState == TX_OFF)
	{
		//UART_writeCancel(uart);
	}

	if (rfState == TX_CW_ON)
	{

	    /* Explicitly configure CW (1) or Modulated (0). Default modulated mode is PRBS-15. */
	    RF_cmdTxTest.config.bUseCw = 1;

	    if (rfHandle != NULL)
	        RF_close(rfHandle);

	    /* In order to achieve +14dBm output power, make sure .txPower = 0xa73f, and
	    that the define CCFG_FORCE_VDDR_HH = 0x1 in ccfg.c */

	    /* Request access to the radio */
	    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);

	    /* Send CMD_FS and wait until it has completed */
	    RF_runCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal);

	    /* Send CMD_TX_TEST which sends forever */
	    cw_cmd = RF_postCmd(rfHandle, (RF_Op*)&RF_cmdTxTest, RF_PriorityNormal, NULL);
	    //result = RF_waitCmd(rfHandle, cw_cmd, RF_EventCmdAborted);

	    rfState = TX_OFF;

	    /* never reaches this */
	    //uart_writePayLoad(RF_cmdPropTx.pPkt, RF_cmdPropTx.pktLen);
	}

	if (rfState == TX_CW_OFF)
	{
		RF_abortCmd(rfHandle, cw_cmd);
    	rfState = TX_OFF;
        /* Close connection to radio */
        RF_close(rfHandle);
        rfHandle = NULL;
	}

    packetCounter = 1;

	while (rfState ==  RX_ON)
	{
        rx_cmd = RF_postCmd(rfHandle, (RF_Op*)&RF_cmdPropRx, RF_PriorityNormal, NULL);
        result = RF_waitCmd(rfHandle, rx_cmd, RF_EventCmdDone);

        if (result == RF_EventCmdDone)
        {
            do {
                /* Toggle pin to indicate RX */
                PIN_setOutputValue(pinHandle, Board_LED4,!PIN_getOutputValue(Board_LED4));

                /* Get current unhandled data entry */
                currentDataEntry = RFQueue_getDataEntry();

                /* Handle the packet data, located at &currentDataEntry->data:
                * - Length is the first byte with the current configuration
                * - Data starts from the second byte */
                packetLength      = *(uint8_t*)(&currentDataEntry->data);
                packetDataPointer = (uint8_t*)(&currentDataEntry->data + 1);

                rssiDataPointer = (uint8_t*)(&currentDataEntry->data + packetLength + 1);
                rssi = (int8_t) *rssiDataPointer;

                /* Check for error */
                if (rxStatistics.nRxNok > prevRcvOK){
                	error = 1;
                }
                else {
                	error = 0;
                }
                prevRcvOK = rxStatistics.nRxNok;

                /* Check sequence */
                rcvSeqNumber = (uint8_t)packetDataPointer[1];
            	if (rcvSeqNumber != 0) { // ignore first seqNumber.
                    if (rcvSeqNumber != (prevRcvSeqNumber + 1)) {
                	    error = 1;
                    }
                }
                prevRcvSeqNumber = rcvSeqNumber;


                /* Print results */
                uart_writeError();
                uart_writeRssi();
                uart_writePayLoad(packetDataPointer, packetLength);


                /* This code block is added to avoid a compiler warning.
                * Normally, an application will reference these variables for
                * useful data. */
                volatile uint8_t dummy;
                dummy = packetLength;
                dummy = packetDataPointer[0];

            } while(RFQueue_nextEntry() == DATA_ENTRY_FINISHED);
        }

#if 0
        /* Exit after all packets are received */
        if (packetCounter++ == tx_numPackets)
        	rfState =  RX_TURN_OFF;
#endif

	}

    if (rfState == RX_TURN_OFF) {
        RF_close(rfHandle);
        rfHandle = NULL;
    	rfState = RX_OFF;
    }

}


void runTxFreqHopping() {
	int rfChannel;

	rfChannel = 0;

    while (txFreqHoppingEnabled) {
    	/* Update the center frequency */
        RF_cmdFs.frequency = fh_868_channels[rfChannel].frequency;
        RF_cmdPropRadioDivSetup.centerFreq = fh_868_channels[rfChannel].frequency;
        RF_cmdFs.fractFreq = fh_868_channels[rfChannel].fractFreq;


        rfState = TX_TURN_ON;
        processRFState();

        rfChannel = (rfChannel + 1) % NUM_CHANNELS;

        /* Print updated frequency */
        freqToString();
        printResponse();

    }

	return;
}

void runRxFreqHopping() {
	int rfChannel;

	rfChannel = 0;

    while (rxFreqHoppingEnabled) {
    	/* Update the center frequency */
        RF_cmdFs.frequency = fh_868_channels[rfChannel].frequency;
        RF_cmdPropRadioDivSetup.centerFreq = fh_868_channels[rfChannel].frequency;
        RF_cmdFs.fractFreq = fh_868_channels[rfChannel].fractFreq;


        rfState = RX_TURN_ON;
        processRFState();

        rfChannel = (rfChannel + 1) % NUM_CHANNELS;

        /* Print updated frequency */
        freqToString();
        printResponse();

    }

	return;
}
