/******************************************************************************
*  Filename:       serialIO.c
*  Revised:        $ $
*  Revision:       $ $
*
*  Description:    Help functions for handling queues
*
*  Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
*
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************/

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

/* XDCtools Header files */
#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/System.h>

#include "serialIO.h"

/* Example/Board Header files */
#include "Board.h"

uart_t uart_s;

//*****************************************************************************
//
//! Intializes Uart port
//!
//! \return output buffer length after bit insertion
//
//*****************************************************************************
void uart_init() {
}

//*****************************************************************************
//
//! Takes input command line
//! And parses into command and the arguments
//! \return output buffer length after bit insertion
//
//*****************************************************************************

void uart_get_command_and_arguments(){
	uint8_t i, currArg;
	char *line_p;

	/* Clear working buffers from previous command data */
	memset(uart_s.command_str, 0, UART_SERIAL_COMMAND_SIZE);
	memset(uart_s.arg[0], 0, UART_ARGUMENT_SIZE);
	memset(uart_s.arg[1], 0, UART_ARGUMENT_SIZE);

	/* use a pointer to fill command and arguments from the line */
	line_p = &uart_s.command_str[0];
	currArg = 0;

	/* Acquire command and arguments from the command line */
	for (i = 0; i <uart_s.lineSize; i++)
	{
		if (uart_s.inputLine[i] != (char) CHAR_SPACE)
		{
			*line_p++ = uart_s.inputLine[i];
		}
		else
		{
		    line_p = &uart_s.arg[currArg][0];
		    currArg++;
		}

	}


	/* Process cmd */
	uart_s.command = 0;
	i = 0;
	while ((uint8_t)uart_s.command_str[i] != 0)
	{
	    /* Compute weighted check sum for easy command mapping */
	    uart_s.command += ((uint8_t) uart_s.command_str[i] - CHAR_LOWERCASE_START) * (i+1);
	    i++;
	}


	/* Process arg1 */
	if(uart_s.arg[0][0] == (char) CHAR_QUESTION_MARK)
	{
		uart_s.commandType = COMMAND_GET;
	}
	else
	{
	    uart_s.commandType = COMMAND_SET;
	}

}

//*****************************************************************************
//
//! Converts input payload to hex and prints to uart.
//
//*****************************************************************************
extern UART_Handle uart;

void uart_writePayLoad(uint8_t *packet, uint8_t length) {
	char output[2];
	uint8_t i;
	uint8_t value;

	UART_write(uart, "bytes: ", 7);

	for(i = 0; i < length; i++) {
		/* convert 8 bit value to hex */
		value = (*packet >> 4);
		if (value < 10)
		    output[0] = value + CHAR_ZERO;
		else
		    output[0] = value + (CHAR_UPPERCASE_START - 9);

		value = (*packet % 16);
		if (value < 10)
		    output[1] = value + CHAR_ZERO;
		else
		    output[1] = value + (CHAR_UPPERCASE_START - 9);

		packet++;

		UART_write(uart, output, 2);

		if (i % 2) {
			/* print space after every 2 bytes for readability */
			output[0] = CHAR_SPACE;
			UART_write(uart, output, 1);
		}


	}
	/* Output a carriage return */
	output[0] = CHAR_LINE_END_1;
	UART_write(uart, output, 1);

}




//*****************************************************************************
//
//! Prints RSSI value to uart.
//
//*****************************************************************************
extern int8_t rssi;

void uart_writeRssi() {

	int8_t i,localRssi;
	int value;

	localRssi = rssi;

	strcpy(uart_s.response, "rssi=     ; ");
	uart_s.responseMsgLen = 12;

	if (rssi < 0) {
		uart_s.response[6] = (char)CHAR_MINUS;
		localRssi = -rssi;
	}

    /* Fill value after decimal point */
    i = 9;
    while(localRssi > 0) {
    	value = localRssi % 10;
    	uart_s.response[i] = value + (char)CHAR_ZERO;
    	localRssi = localRssi / 10;
    	i--;
    }

    UART_write(uart, uart_s.response, uart_s.responseMsgLen);

}

//*****************************************************************************
//
//! Prints Error value to uart.
//
//*****************************************************************************
extern int8_t error;

void uart_writeError() {

	strcpy(uart_s.response, "error=  ; ");
	uart_s.responseMsgLen = 10;
	uart_s.response[7] = (char)(CHAR_ZERO);

	if (error == 1) {
		uart_s.response[7] = (char)(CHAR_ZERO + 1);
	}

    UART_write(uart, uart_s.response, uart_s.responseMsgLen);

}


//*****************************************************************************
//
//! Prints usage information
//
//*****************************************************************************

void uart_usage() {
	char nextLine;

    nextLine = (char)CHAR_LINEFEED;

	strcpy(uart_s.response, "Usage information: ");
	uart_s.responseMsgLen = 19;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    start_tx");
	uart_s.responseMsgLen = 12;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    stop_tx");
	uart_s.responseMsgLen = 11;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    start_rx");
	uart_s.responseMsgLen = 12;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    stop_rx");
	uart_s.responseMsgLen = 11;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    absolute_frequency ? or GMMMKKKHHH");
	uart_s.responseMsgLen = 38;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    tx_repeat <NumPackets> <time gap in ms>");
	uart_s.responseMsgLen = 43;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    get_radio_config");
	uart_s.responseMsgLen = 20;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    cw on/off");
	uart_s.responseMsgLen = 13;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    tx_packet ?");
	uart_s.responseMsgLen = 15;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    tx_power ?/X");
	uart_s.responseMsgLen = 16;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    strcpy(uart_s.response, "    usage");
	uart_s.responseMsgLen = 9;
    UART_write(uart, uart_s.response, uart_s.responseMsgLen);
    UART_write(uart, &nextLine, 1);

    uart_s.responseMsgLen = 0;
    UART_write(uart, &nextLine, 1);

}
