/******************************************************************************
*  Filename:       serialIO.h
*  Revised:        $ $
*  Revision:       $ $
*
*  Description:    Help functions for handling queues
*
*  Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
*
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************/

//*****************************************************************************
//
//! \addtogroup rfqueue_api
//! @{
//
//*****************************************************************************

#ifndef SERIAL_IO_H
#define SERIAL_IO_H

#include <ti/drivers/UART.h>

#define UART_SERIAL_LINE_SIZE                 256
#define UART_SERIAL_COMMAND_SIZE              24
#define UART_ARGUMENT_SIZE                    16
#define NUM_COMMANDS                          16

/* ASCII values of some useful keys */
#define CHAR_LINEFEED                         0x0A
#define CHAR_LINE_END_1                       0x0D   // Enter
#define CHAR_LINE_END_2                       0x03   // Enter on numpad
#define CHAR_SPACE                            0x20
#define CHAR_LOWERCASE_START                  0x60
#define CHAR_QUESTION_MARK                    0x3F
#define CHAR_ZERO                             0x30
#define CHAR_UPPERCASE_START                  0x40
#define CHAR_MINUS                            0x2D

/* Weighted checksum value for each command */
/* Use for integer representation of every command for easy comparison */
/* against input command string */
#define ABSOLUTE_FREQUENCY                   2112
#define TX_REPEAT                            481
#define START_TX                             560
#define STOP_TX                              451
#define START_RX                             546
#define STOP_RX                              439
#define RADIO_STATE                          694
#define TX_POWER                             521
#define TX_DATARATE                          621
#define RX_DATARATE                          619
#define TX_BIT_SHAPING                       1029
#define FREQ_DEV                             355
#define TX_BITMODE                           535
#define RX_BITMODE                           533
#define TX_PACKET                            449
#define RX_FILTER_BW                         834
#define CW                                   49
#define GET_RADIO_CONFIG                     1120
#define GET_RADIO_REGISTER                   1883
#define WRITE_REGISTER_FUNCTION              3258
#define RESET                                205
#define USAGE                                115

#if 0
const char commandList[NUM_COMMANDS] = {
    "absolute_frequency",
    "tx_repeat",
    "start_tx",
    "stop_tx",
    "start_rx",
    "stop_rx",
    "radio_state",
    "tx_power",
    "tx_datarate",
    "rx_datarate",
    "tx_bit_shaping",
    "freq_dev",
    "tx_bitmode",
    "rx_bitmode",
    "tx_packet",
    "rx_filter_bw",
    "cw",
    "get_radio_config",
    "get_radio_register",
    "write_register_function",
    "reset",
    "start_tx_fh",
    "start_rx_fh",
    "stop_tx_fh",
    "stop_rx_fh"

};
#endif

//! \addtogroup Uart
//! @{
//! Uart object structure

enum queryFlag {
	COMMAND_SET,
	COMMAND_GET,
};


typedef struct {
   char              inputLine[UART_SERIAL_LINE_SIZE];               //!<        Input command line until carriage return
   char              command_str[UART_SERIAL_COMMAND_SIZE];     //!<        Input command sting
   char              arg[2][UART_ARGUMENT_SIZE];                //!<        Argument array
   uint8_t           lineSize;                                  //!<        Size of input command line
   uint16_t          command;                                   //!<        Input command
   char              response[UART_SERIAL_LINE_SIZE];           //!<        Response output string
   uint8_t           responseMsgLen;                            //!<        Response output string length
   enum queryFlag    commandType;                               //!<        Query type or set type command
}uart_t;

extern uart_t uart_s;

//*****************************************************************************
//
// API Overview
//
//*****************************************************************************
// void uart_init(uart_s *uart_p);
// void uart_get_command_and_arguments(uart_s *uart_p);


//*****************************************************************************
//
// API Functions and prototypes
//
//*****************************************************************************

//*****************************************************************************
//
//! Intializes Uart port
//! \return output buffer length after bit insertion
//
//*****************************************************************************
void uart_init();


//*****************************************************************************
//
//! Removes the start bit and stop bit before and after every byte.
//! Also flips the 8 bits of a byte to suit LUTRON's requirements
//! Tail end zero bits in the last residual byte will be deleted
//!
//! \param *input is the pointer to the input buffer with frame controller bits
//! \param *output is the pointer to the output buffer with no frame controller bits
//!
//! This function gets the FIFO level at which transmit and receive interrupts
//! are generated.
//!
//! \return output buffer length after bit insertion
//
//*****************************************************************************


//*****************************************************************************
//
//! Takes input command line
//! And parses into command and the arguments
//! The command and argument strings are later processed
//
//*****************************************************************************


void uart_get_command_and_arguments();

//*****************************************************************************
//
//! Converts input payload to hex and prints to uart.
//
//*****************************************************************************

void uart_writePayLoad(uint8_t *packet, uint8_t length);


//*****************************************************************************
//
//! Prints RSSI value to uart.
//
//*****************************************************************************
void uart_writeRssi();

//*****************************************************************************
//
//! Prints Error to uart.
//
//*****************************************************************************
void uart_writeError();


//*****************************************************************************
//
//! Print usage.
//
//*****************************************************************************

void uart_usage();

//*****************************************************************************
//
// Mark the end of the C bindings section for C++ compilers.
//
//*****************************************************************************
#ifdef __cplusplus
}
#endif

#endif //  __UART_H__

//*****************************************************************************
//
//! Close the Doxygen group.
//! @}
//
//*****************************************************************************
